﻿Imports System.ComponentModel

Public Class ProgressCircle
    Inherits UserControl

    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        '- initial values when inserted to the window
        Width = 100
        Height = 100
        HorizontalAlignment = HorizontalAlignment.Left
        VerticalAlignment = VerticalAlignment.Top
        Margin = New Thickness(240, 160, 0, 0)
        ArcAngle = 60

    End Sub

    Private UpdatingArcAngle As Boolean
    Private UpdatingProgressValue As Boolean

#Region "Hide some Properties"
    ' hide unused Properties of UserControl in designer
    <Browsable(False)>
    Public Overloads Property Foreground As Brush
    <Browsable(False)>
    Public Overloads Property Background As Brush
    <Browsable(False)>
    Public Overloads Property BorderBrush As Brush
#End Region

#Region "Appearance"

    Private Shared ReadOnly DefaultBackFillBrush As Brush = New SolidColorBrush(Color.FromArgb(&HFF, &HF0, &HF0, &HF0))

    'Private Shared ReadOnly FillPropertyMetadata As New FrameworkPropertyMetadata(Brushes.Gray, FrameworkPropertyMetadataOptions.AffectsRender, New PropertyChangedCallback(AddressOf OnArcAngleChanged), New CoerceValueCallback(AddressOf CoerceArcAngle))
    Private Shared ReadOnly FillPropertyMetadata As New FrameworkPropertyMetadata(Brushes.LightGreen, FrameworkPropertyMetadataOptions.AffectsRender)
    Public Shared ReadOnly FillProperty As DependencyProperty = DependencyProperty.Register("Fill", GetType(Brush), GetType(ProgressCircle), FillPropertyMetadata)

    '<Description("Ellipse Fill"), Category("Progress Circle")>   ' appears in VS Properties
    Public Property Fill As Brush
        Get
            Return CType((GetValue(FillProperty)), Brush)
        End Get
        Set(ByVal value As Brush)
            SetValue(FillProperty, value)
        End Set
    End Property

    Private Shared ReadOnly StrokePropertyMetadata As New FrameworkPropertyMetadata(Brushes.Black, FrameworkPropertyMetadataOptions.AffectsRender)
    Public Shared ReadOnly StrokeProperty As DependencyProperty = DependencyProperty.Register("Stroke", GetType(Brush), GetType(ProgressCircle), StrokePropertyMetadata)

    '<Description("Ellipse Fill"), Category("Progress Circle")>   ' appears in VS Properties
    Public Property Stroke As Brush
        Get
            Return CType((GetValue(StrokeProperty)), Brush)
        End Get
        Set(ByVal value As Brush)
            SetValue(StrokeProperty, value)
        End Set
    End Property

    Private Shared ReadOnly StrokeThicknessPropertyMetadata As New FrameworkPropertyMetadata(1.0, FrameworkPropertyMetadataOptions.AffectsRender)
    Public Shared ReadOnly StrokeThicknessProperty As DependencyProperty = DependencyProperty.Register("StrokeThickness", GetType(Double), GetType(ProgressCircle), StrokeThicknessPropertyMetadata)

    <Description("Ellipse Stroke"), Category("Progress Circle")>   ' appears in VS Properties
    Public Property StrokeThickness As Double
        Get
            Return CDbl(GetValue(StrokeThicknessProperty))
        End Get
        Set(ByVal value As Double)
            SetValue(StrokeThicknessProperty, value)
        End Set
    End Property

    Private Shared ReadOnly BackFillPropertyMetadata As New FrameworkPropertyMetadata(DefaultBackFillBrush, FrameworkPropertyMetadataOptions.AffectsRender)
    Public Shared ReadOnly BackFillProperty As DependencyProperty = DependencyProperty.Register("BackFill", GetType(Brush), GetType(ProgressCircle), BackFillPropertyMetadata)

    '<Description("Ellipse Fill"), Category("Progress Circle")>   ' appears in VS Properties
    Public Property BackFill As Brush
        Get
            Return CType((GetValue(BackFillProperty)), Brush)
        End Get
        Set(ByVal value As Brush)
            SetValue(BackFillProperty, value)
        End Set
    End Property


#End Region

#Region "Angle and Value"
    'Private Const StartAngle As Single = 270.0          ' from 3 o'clock, clockwise (270 = 12 o'clock)

    Private Shared ReadOnly StartAnglePropertyMetadata As New FrameworkPropertyMetadata(270.0F, FrameworkPropertyMetadataOptions.AffectsRender, New PropertyChangedCallback(AddressOf OnStartAngleChanged), New CoerceValueCallback(AddressOf CoerceStartAngle))
    Public Shared ReadOnly StartAngleProperty As DependencyProperty = DependencyProperty.Register("StartAngle", GetType(Single), GetType(ProgressCircle), StartAnglePropertyMetadata)
    <Description("Start of the Arc, relative to three o'clock, clockwise"), Category("Progress Circle")>   ' appears in VS Properties
    Public Property StartAngle As Single
        Get
            Return CSng(GetValue(StartAngleProperty))
        End Get
        Set(ByVal value As Single)
            SetValue(StartAngleProperty, value)
        End Set
    End Property
    Private Shared Function CoerceStartAngle(ByVal depObj As DependencyObject, ByVal baseVal As Object) As Object
        Dim angle As Single = CSng(baseVal)
        If angle > 360 Then
            angle = 360
        ElseIf angle < 0.0 Then
            angle = 0.0
        End If
        Return angle
    End Function
    Private Shared Sub OnStartAngleChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)

        'Dim value As Single

        'value = CSng((CSng(d.GetValue(ArcAngleProperty)) / 3.6))
        'd.SetCurrentValue(ProgressValueProperty, value)

    End Sub

    Private Shared ReadOnly ArcAnglePropertyMetadata As New FrameworkPropertyMetadata(0.0F, FrameworkPropertyMetadataOptions.AffectsRender, New PropertyChangedCallback(AddressOf OnArcAngleChanged), New CoerceValueCallback(AddressOf CoerceArcAngle))
    Public Shared ReadOnly ArcAngleProperty As DependencyProperty = DependencyProperty.Register("ArcAngle", GetType(Single), GetType(ProgressCircle), ArcAnglePropertyMetadata)
    <Description("The current Angle for Arc"), Category("Progress Circle")>   ' appears in VS Properties
    Public Property ArcAngle As Single
        Get
            Return CSng(GetValue(ArcAngleProperty))
        End Get
        Set(ByVal value As Single)
            SetValue(ArcAngleProperty, value)
        End Set
    End Property
    Private Shared Function CoerceArcAngle(ByVal depObj As DependencyObject, ByVal baseVal As Object) As Object
        Dim angle As Single = CSng(baseVal)
        If angle > 360 Then
            angle = 360
        ElseIf angle < 0.0 Then
            angle = 0.0
        End If
        Return angle
    End Function
    Private Shared Sub OnArcAngleChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        Dim control As ProgressCircle = CType(d, ProgressCircle)
        control.UpdatingArcAngle = True

        If control.UpdatingProgressValue = False Then
            Dim value As Single
            value = CSng((CSng(d.GetValue(ArcAngleProperty)) / 3.6))
            d.SetCurrentValue(ProgressValueProperty, value)
        End If

        control.UpdatingArcAngle = False
    End Sub

    Private Shared ReadOnly ProgressValuePropertyMetadata As New FrameworkPropertyMetadata(0.0F, FrameworkPropertyMetadataOptions.AffectsRender, New PropertyChangedCallback(AddressOf OnProgressValueChanged), New CoerceValueCallback(AddressOf CoerceProgressValue))
    Public Shared ProgressValueProperty As DependencyProperty = DependencyProperty.Register("ProgressValue", GetType(Single), GetType(ProgressCircle), ProgressValuePropertyMetadata)

    <Description("The current Progress Value"), Category("Progress Circle")>   ' appears in VS Properties
    Public Property ProgressValue As Single
        Get
            Return CSng(GetValue(ProgressValueProperty))
        End Get
        Set(value As Single)
            SetValue(ProgressValueProperty, value)
        End Set
    End Property
    Private Shared Function CoerceProgressValue(ByVal depObj As DependencyObject, ByVal baseVal As Object) As Object
        Dim value As Single = CSng(baseVal)
        If value > 100 Then
            value = 100
        ElseIf value < 0.0 Then
            value = 0.0
        End If
        Return value
    End Function
    Private Shared Sub OnProgressValueChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        Dim control As ProgressCircle = CType(d, ProgressCircle)
        control.UpdatingProgressValue = True

        If control.UpdatingArcAngle = False Then
            Dim value As Single
            value = CSng(CSng(d.GetValue(ProgressValueProperty)) * 3.6)
            d.SetCurrentValue(ArcAngleProperty, value)
        End If

        control.UpdatingProgressValue = False
    End Sub

#End Region

#Region "Render"
    ' for description see below
    Protected Overrides Sub OnRender(ByVal dc As DrawingContext)

        If BackFill IsNot Nothing Then
            Dim myEllipseGeometry As New EllipseGeometry()
            myEllipseGeometry.Center = New Point(RenderSize.Width / 2.0, RenderSize.Height / 2.0)
            myEllipseGeometry.RadiusX = (RenderSize.Width - StrokeThickness) / 2.0
            myEllipseGeometry.RadiusY = (RenderSize.Height - StrokeThickness) / 2.0

            dc.DrawGeometry(BackFill, New Pen(), myEllipseGeometry)
        End If

        '---

        Dim maxWidth = Math.Max(0.0, RenderSize.Width - Ellipse1.StrokeThickness)
        Dim maxHeight = Math.Max(0.0, RenderSize.Height - Ellipse1.StrokeThickness)

        Dim xStart = maxWidth / 2.0 * Math.Cos((StartAngle) * Math.PI / 180.0)
        Dim yStart = maxHeight / 2.0 * Math.Sin((StartAngle) * Math.PI / 180.0)

        Dim xEnd = maxWidth / 2.0 * Math.Cos((StartAngle + ArcAngle) * Math.PI / 180.0)
        Dim yEnd = maxHeight / 2.0 * Math.Sin((StartAngle + ArcAngle) * Math.PI / 180.0)

        'Dim geom = New StreamGeometry

        'If ArcAngle = 0 Then Return geom   ' suppress start-line

        If ArcAngle < 360 Then
            Dim geom = New StreamGeometry

            Using ctx = geom.Open
                ctx.BeginFigure(New Point(RenderSize.Width / 2.0 + xStart, RenderSize.Height / 2.0 + yStart), True, True)
                ctx.ArcTo(New Point(RenderSize.Width / 2.0 + xEnd, RenderSize.Height / 2.0 + yEnd), New Size(maxWidth / 2.0, maxHeight / 2), 0.0, ArcAngle > 180, SweepDirection.Clockwise, True, False)
                ctx.LineTo(New Point(RenderSize.Width / 2.0, RenderSize.Height / 2.0), True, False)
            End Using

            dc.DrawGeometry(Fill, New Pen(Stroke, StrokeThickness), geom)
        Else
            Dim myEllipseGeometry As New EllipseGeometry()
            myEllipseGeometry.Center = New Point(RenderSize.Width / 2.0, RenderSize.Height / 2.0)
            myEllipseGeometry.RadiusX = (RenderSize.Width - StrokeThickness) / 2.0
            myEllipseGeometry.RadiusY = (RenderSize.Height - StrokeThickness) / 2.0

            dc.DrawGeometry(Fill, New Pen(Stroke, StrokeThickness), myEllipseGeometry)
        End If

    End Sub

#End Region

#Region "Geometry and render description"

    ' In this example, the angle 0 is at 3 o'clock and increases clockwise
    '
    '--- Geometry ---
    ' from UI RenderSize we're getting the designed size of the control (f.e. 100/100)
    ' the size of the circle is size of control - StrokeThickness (= maxWidth,maxHeight)
    ' radius of the circle is (size of circle) / 2
    '           
    '               r    / ¦
    '                 /    ¦ y
    '              /       ¦  
    '            <.........¦ C(90)
    '             A    x
    '
    ' looking at the triangle where angle C is 90 degrees and side(line) r is = radius:
    ' with angle A and side(line) r, we can calculate x and y of a point on the circle
    ' -> calculating start- and end- point of the arc.
    '   
    '  * Math.PI / 180.0 to convert degrees to radians
    '
    '--- example with values ---
    '
    ' Width = Height (square, circle)
    '
    ' RenderSize.Width	100
    ' StrokeThickness   1
    '
    ' maxWidth 		    99
    '
    ' StartAngle  		270     (at 12 o'clock)
    '
    ' xStart            0
    ' yStart	        -49.5
    '
    ' ArcAngle          90
    '
    ' xEnd			    49.5
    ' yEnd			    0
    '---
    '
    ' 
    '--- Rendering ---
    '
    ' when ArcAngle = 360 degrees then we have a full circle an can directly draw an ellipse
    '
    ' else:
    ' we're beginning a new StreamGeometry at xStart, yStart on the circle
    ' drawing an arc to xEnd, yEnd on the circle
    ' drawing a line to the midpoint of the circle
    ' at BeginFigure 'isClosed' was set to true, so the figure closes itself to the the first point (xStart, yStart)
    ' now the geometry can be drawn with the given fill-brush and the given stroke and StrokeThickness

#End Region

End Class
